=begin
###
Copyright 2011 TIG (c)
Permission to use, copy, modify, and distribute this software for any 
purpose, and currently without fee, is hereby granted, provided that  
this text and the above copyright (c) notice appear in all copies.
This software is provided "as is" and without any express or implied 
warranties, including, without limitation, the implied warranties of 
merchantability and fitness for a particular purpose.
###
Name: TIG-dropmaterialsontoinstances.rb
###
Menu: 'Plugins' > 'Drop Materials onto Instances'
###
Usage:
Make a flat set of faces [it's easier to manipulate/scale if a group or 
a component] with colors to represent the materials required to be 
dropped.
For example draw/make the text logo and color it, or use 'ImageTrimmer' 
on a transparent png to  get the cutout faces and then repaint those 
with new materials in simple colors as desired, or try ThomThom's tool 
'Bitmap to Mesh'.
Place this set of faces directly over the groups/instances to be 
processed.
Select all of the groups/instances to be processed and activate the 
tool.
A count down is shown in the status bar.
Every group/instance with its center below a face will take the material 
of that face; so any default material faces within them will appear with 
those materials.  Other groups/instances remain unaffected.
The smaller the groups/instances relative to the faces above the less 
'pixelated' the result will be...
It is one step undo able.
###
Donations: PayPal.com to info @ revitrev.org
###
Version:
1.0 20111015 First issue.
1.1 20111019 Error in menu coding fixed.
###
=end
require 'sketchup.rb'
module TIG
  unless file_loaded?(File.basename(__FILE__))
    UI.menu('Plugins').add_item('Drop Materials onto Instances'){self.dropmaterialsontoinstances()}
  end
  file_loaded(File.basename(__FILE__))
  ###
def self.dropmaterialsontoinstances()
  ###
  model=Sketchup.active_model
  ss=model.selection
  insts=[]
  ss.each{|e|insts << e if e.class==Sketchup::Group or e.class==Sketchup::ComponentInstance}
  ctr=1
  tot=insts.length
  model.start_operation('Drop Materials onto Instances')
  insts.each{|e|
    Sketchup.status_text="Drop Materials onto Instances: "+ctr.to_s+" of "+tot.to_s
    ctr+=1
    bb=e.bounds
    max=bb.max
    cen=bb.center
    cen.z=max.z
    rt=model.raytest([cen, Z_AXIS])
    next if not rt
    face=rt[1][-1]
    next if not face.class==Sketchup::Face
    e.material=face.material
  }
  model.commit_operation
end

end